/*
 * This file is part of the PhotoEditor Software Development Kit.
 *
 * Copyright (C) 2017 9elements GmbH <contact@9elements.com>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, without
 * modification, are permitted provided that the following license agreement
 * is approved and a legal/financial contract was signed by the user.
 *
 * The license agreement can be found under the following link:
 *
 * https://www.photoeditorsdk.com/LICENSE.txt
 */

package com.ahazou.photo_editor_sdk.instagram_ui;

import android.animation.AnimatorSet;
import android.animation.ObjectAnimator;
import android.app.Activity;
import android.app.ProgressDialog;
import android.content.Intent;
import android.content.res.Configuration;
import android.graphics.Bitmap;
import android.net.Uri;
import android.os.Bundle;
import android.provider.MediaStore;
import android.util.Log;
import android.view.View;
import android.widget.ImageButton;
import android.widget.LinearLayout;
import android.widget.Toast;

import androidx.annotation.IntDef;
import androidx.annotation.MainThread;
import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import com.ahazou.photo_editor_sdk.PhotoEditorSdkPlugin;
import com.ahazou.photo_editor_sdk.R;
import com.ahazou.photo_editor_sdk.instagram_ui.panel.BrushPanel;
import com.ahazou.photo_editor_sdk.instagram_ui.panel.LogoPanel;
import com.ahazou.photo_editor_sdk.instagram_ui.panel.OnPanelCloseListener;
import com.ahazou.photo_editor_sdk.instagram_ui.panel.StickerPanel;
import com.ahazou.photo_editor_sdk.instagram_ui.panel.TextPanel;
import com.ahazou.photo_editor_sdk.instagram_ui.widget.ConfirmPopupWindow;
import com.ahazou.photo_editor_sdk.utils.CustomImageStickerAsset;
import com.ahazou.photo_editor_sdk.utils.Utils;
import com.ahazou.services.ApiClient;
import com.ahazou.services.ApiService;
import com.ahazou.services.LogoToResponse;

import ly.img.android.IMGLYEvents;
import ly.img.android.pesdk.annotations.OnEvent;
import ly.img.android.pesdk.annotations.StateEvents;
import ly.img.android.pesdk.backend.model.state.BrushSettings;
import ly.img.android.pesdk.backend.model.state.EditorLoadSettings;
import ly.img.android.pesdk.backend.model.state.EditorShowState;
import ly.img.android.pesdk.backend.model.state.HistoryState;
import ly.img.android.pesdk.backend.model.state.LayerListSettings;
import ly.img.android.pesdk.backend.model.state.LoadSettings;
import ly.img.android.pesdk.backend.model.state.ProgressState;
import ly.img.android.pesdk.backend.model.state.SaveSettings;
import ly.img.android.pesdk.backend.model.state.layer.ImageStickerLayerSettings;
import ly.img.android.pesdk.backend.model.state.layer.TextLayerSettings;
import ly.img.android.pesdk.backend.model.state.manager.StateHandler;
import ly.img.android.pesdk.backend.views.EditorPreview;
import ly.img.android.pesdk.ui.activity.ImgLyActivity;
import ly.img.android.pesdk.ui.activity.ImgLyIntent;
import ly.img.android.pesdk.ui.utils.PermissionRequest;
import ly.img.android.pesdk.ui.widgets.ConfirmPopupView;
import ly.img.android.pesdk.utils.OrientationSensor;
import ly.img.android.pesdk.utils.ThreadUtils;
import ly.img.android.pesdk.utils.TimeOut;
import okhttp3.MediaType;
import okhttp3.MultipartBody;
import okhttp3.RequestBody;
import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.util.List;

@StateEvents
public class InstagramUIActivity extends ImgLyActivity implements OnPanelCloseListener,
        TimeOut.Callback {

    public static int SELECT_IMAGE = 0x1;

    private static final int ANIMATION_DURATION = 180;

    private static final int EDITOR_MODE_MENU = 0;
    private static final int EDITOR_MODE_LOGO = 1;
    private static final int EDITOR_MODE_STICKER = 2;
    private static final int EDITOR_MODE_BRUSH = 3;
    private static final int EDITOR_MODE_TEXT = 4;
    private static final int EDITOR_MODE_EDIT_TEXT = 5;

    @IntDef({EDITOR_MODE_MENU, EDITOR_MODE_LOGO, EDITOR_MODE_STICKER,
            EDITOR_MODE_BRUSH, EDITOR_MODE_TEXT, EDITOR_MODE_EDIT_TEXT})
    @Retention(RetentionPolicy.SOURCE)
    private @interface EDITOR_MODE {}

    @EDITOR_MODE
    private int editorMode = 0;

    private HistoryState historyState;
    private EditorShowState editorState;
    private BrushSettings brushSettings;

    private LayerListSettings layerListSettings;

    private View rootView;
    private EditorPreview editorPreviewView;

    private LinearLayout menuPanel;
    private TextPanel textPanel;
    private BrushPanel brushPanel;
    private LogoPanel logoPanel;
    private StickerPanel stickerPanel;

    private RecyclerView textFontRecyclerView;

    private View editStickerPanel;
    private View undoRedoPanel;

    private ImageButton undoButton;
    private ImageButton redoButton;

    private TimeOut timeOut = new TimeOut(null);

    private boolean canvasTouched = false;

    AnimatorSet animatorSet;


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.instagram_ui_activity);
        getStateHandler().registerSettingsEventListener(this);

        editorState = getStateHandler().getStateModel(EditorShowState.class);
        historyState = getStateHandler().getStateModel(HistoryState.class);
        brushSettings = getStateHandler().getStateModel(BrushSettings.class);
        brushSettings.saveInitState();

        layerListSettings = getStateHandler().getStateModel(LayerListSettings.class);
        layerListSettings.setSelected(null);

        timeOut.addCallback(this);

        initViews();
    }

    private void initViews() {
        // Root element
        rootView = findViewById(R.id.rootView);
        // Content image
        editorPreviewView = findView(R.id.editorImageView);

        // Bottom menu
        menuPanel = findViewById(R.id.iui_main_ui);

        // Editions
        logoPanel = findViewById(R.id.iui_logo_ui);
        stickerPanel = findViewById(R.id.iui_sticker_ui);
        brushPanel = findViewById(R.id.iui_brush_ui);
        textPanel = findViewById(R.id.iui_text_ui);

        logoPanel.setOnPanelCloseListener(this);
        stickerPanel.setOnPanelCloseListener(this);
        brushPanel.setOnPanelCloseListener(this);
        textPanel.setOnPanelCloseListener(this);

        editStickerPanel = findViewById(R.id.edit_sticker_panel);
        undoRedoPanel = findViewById(R.id.undo_redo_panel);

        textFontRecyclerView = findView(R.id.text_font_recycler_view);

        logoPanel.setActivity(this);

        logoPanel.setAdapter(PhotoEditorSdkPlugin.arrLogos);
        stickerPanel.setAdapter(PhotoEditorSdkPlugin.arrStickers);

        // Sticker edition
        ImageButton deleteStickerButton = findViewById(R.id.delete_sticker_button);
        deleteStickerButton.setOnClickListener(v -> layerListSettings.removeLayer(layerListSettings.getSelected()));

        /*InstagramFontAdapter_ fontAdapter = new InstagramFontAdapter_(fontConfig -> {
            TextPanel textPanel = findViewById(EditorMode.TEXT.editorPanelId);
            textPanel.updateFont(fontConfig);
            openEditStickerPanel();
            closeFontPicker();
        });
        List<FontConfig> fonts = new ArrayList<>();
        for (FontConfigInterface fontConfigInterface : getStateHandler().getStateModel(PESDKConfig.class).getFontConfig()) {
            if (fontConfigInterface instanceof FontConfig) {
                fonts.add((FontConfig) fontConfigInterface);
            }
        }
        fontAdapter.submitList(fonts);
        textFontRecyclerView.setLayoutManager(new LinearLayoutManager(this, LinearLayoutManager.HORIZONTAL, false));
        textFontRecyclerView.setAdapter(fontAdapter);*/

        // Close
        findViewById(R.id.iui_close_button).setOnClickListener(v -> onCloseClicked());
        // Export
        findViewById(R.id.iui_save_button).setOnClickListener(v -> onSaveClicked());
        // Logo Panel
        findViewById(R.id.editor_mode_logo)
                .setOnClickListener(v -> switchEditorMode(EDITOR_MODE_LOGO));
        // Sticker Panel
        findViewById(R.id.editor_mode_sticker)
                .setOnClickListener(v -> switchEditorMode(EDITOR_MODE_STICKER));
        // Text Panel
        findViewById(R.id.editor_mode_text)
                .setOnClickListener(v -> switchEditorMode(EDITOR_MODE_TEXT));

        undoButton = findViewById(R.id.undo_button);
        undoButton.setOnClickListener((v) -> {
            historyState.undo(0);
            undoButton.setImageResource(!historyState.hasUndoState(0) ? R.drawable.imgly_icon_undo_disabled : R.drawable.imgly_icon_undo_enabled);
            redoButton.setImageResource(!historyState.hasRedoState(0) ? R.drawable.imgly_icon_redo_disabled : R.drawable.imgly_icon_redo_enabled);
        });

        redoButton = findViewById(R.id.redo_button);
        redoButton.setOnClickListener((v) -> {
            historyState.redo(0);
            undoButton.setImageResource(!historyState.hasUndoState(0) ? R.drawable.imgly_icon_undo_disabled : R.drawable.imgly_icon_undo_enabled);
            redoButton.setImageResource(!historyState.hasRedoState(0) ? R.drawable.imgly_icon_redo_disabled : R.drawable.imgly_icon_redo_enabled);
        });
    }

    private void switchEditorMode(@EDITOR_MODE int editorMode) {
        this.editorMode = editorMode;

        switch (editorMode) {
            case EDITOR_MODE_MENU:
                getLayerListSettings().setSelected(null);
                break;
            case EDITOR_MODE_LOGO:
            case EDITOR_MODE_STICKER:
            case EDITOR_MODE_BRUSH:
            case EDITOR_MODE_TEXT:
                deselectSticker();
                break;
            case EDITOR_MODE_EDIT_TEXT:
                // do not deselect sticker
                break;
        }

        logoPanel.switchVisibility(editorMode == EDITOR_MODE_LOGO);
        stickerPanel.switchVisibility(editorMode == EDITOR_MODE_STICKER);
        brushPanel.switchVisibility(editorMode == EDITOR_MODE_BRUSH);
        textPanel.switchVisibility(editorMode == EDITOR_MODE_TEXT || editorMode == EDITOR_MODE_EDIT_TEXT);
        // menuPanel.setVisibility(editorMode == EDITOR_MODE_MENU ? View.VISIBLE : View.GONE);
    }

    private void displayHistoryPanel(int visibility) {
        undoRedoPanel.setVisibility(visibility);
        if (View.VISIBLE == visibility) {
            undoButton.setImageResource(!historyState.hasUndoState(0) ? R.drawable.imgly_icon_undo_disabled : R.drawable.imgly_icon_undo_enabled);
            redoButton.setImageResource(!historyState.hasRedoState(0) ? R.drawable.imgly_icon_redo_disabled : R.drawable.imgly_icon_redo_enabled);
        }
    }

    @OnEvent(IMGLYEvents.LayerListSettings_SELECTED_LAYER)
    protected void onStickerSelect(LayerListSettings settings) {
        LayerListSettings.LayerSettings selected = settings.getSelected();
        if (selected != null) {
            settings.bringLayerToFront(selected);
        }
    }

    @Override
    public void onPanelClose(View closedPanel) {
        switchEditorMode(EDITOR_MODE_MENU);
    }

    private void closeEditor() {
        final Intent result = new Intent();
        result.putExtra(ImgLyIntent.SETTINGS_LIST, getStateHandler().createSettingsListDump());
        Uri input = getStateHandler().getSettingsModel(EditorLoadSettings.class).getImageSource();
        if (input != null && input.getScheme() != null) {
            result.putExtra(ImgLyIntent.SOURCE_IMAGE_PATH, isFileSchema(input) ? input.getPath() : input.toString());
        }
        result.putExtra(ImgLyIntent.SOURCE_IMAGE_URI, getStateHandler().getSettingsModel(EditorLoadSettings.class).getImageSource());
        setResult(RESULT_CANCELED, result);
        finish();
    }

    @MainThread
    protected void onCloseClicked() {
        if (!getStateHandler().getSettingsModel(EditorLoadSettings.class).isDeleteProtectedSource()
                || getStateHandler().hasChanges()
        ) {
            new ConfirmPopupWindow(this).setListener(accepted -> {
                if (accepted) {
                    closeEditor();
                }
            }).show(rootView);
        } else {
            closeEditor();
        }
    }

    @MainThread
    protected void onSaveClicked() {
        StateHandler stateHandler = getStateHandler();
        stateHandler.getStateModel(ProgressState.class).notifyExportStart();

        LoadSettings loadSettings = stateHandler.getStateModel(LoadSettings.class);
        SaveSettings saveSettings = stateHandler.getStateModel(SaveSettings.class);

        if (saveSettings.isExportNecessary()) {
            saveSettings.saveImage((stateHandler1, inputPath, outputPath) -> {
                SaveSettings saveSettings1 = stateHandler1.getStateModel(SaveSettings.class);
                onImageReady(inputPath, outputPath, saveSettings1.getSavePolicy());
            });
        } else {
            Uri inputPath = loadSettings.getSource();
            onImageReady(inputPath, inputPath, saveSettings.getSavePolicy());
        }

    }

    @Override
    public void onBackPressed() {
        if (!ConfirmPopupView.onBackPressed(rootView)) {
            onCloseClicked();
        }
    }

    @Override
    public void onConfigurationChanged(Configuration newConfig) {
        super.onConfigurationChanged(newConfig);
    }

    @Override
    protected void onResume() {
        super.onResume();
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        PermissionRequest.onRequestPermissionsResult(requestCode, permissions, grantResults);
    }

    @Override
    protected void onPause() {
        super.onPause();
        OrientationSensor.getInstance().stop();
    }

    public void onImageReady(final Uri input, final Uri output, final SaveSettings.SavePolicy savePolicy) {
        ThreadUtils.getWorker().addTask(new ThreadUtils.SequencedThreadRunnable("OnResultSaving") {
            @Override
            public void run() {
                final Intent result = new Intent();
                result.putExtra(ImgLyIntent.SETTINGS_LIST, getStateHandler().createSettingsListDump());

                switch (savePolicy) {
                    case KEEP_SOURCE_AND_CREATE_ALWAYS_OUTPUT:
                    case KEEP_SOURCE_AND_CREATE_OUTPUT_IF_NECESSARY:
                        result.putExtra(ImgLyIntent.SOURCE_IMAGE_PATH, isFileSchema(input) ? input.getPath() : input.toString());
                        result.putExtra(ImgLyIntent.SOURCE_IMAGE_URI, input);
                        result.putExtra(ImgLyIntent.RESULT_IMAGE_PATH, isFileSchema(output) ? output.getPath() : output.toString());
                        result.putExtra(ImgLyIntent.RESULT_IMAGE_URI, output);

                        break;
                    case RETURN_ALWAYS_ONLY_OUTPUT:
                    case RETURN_SOURCE_OR_CREATE_OUTPUT_IF_NECESSARY:
                        result.putExtra(ImgLyIntent.RESULT_IMAGE_PATH, isFileSchema(output) ? output.getPath() : output.toString());
                        result.putExtra(ImgLyIntent.RESULT_IMAGE_URI, output);
                        if (input != null) {
                            File file = new File(input.getPath());
                            if (file.exists()) {
                                //noinspection ResultOfMethodCallIgnored
                                file.delete();
                            }
                        }

                    case RENDER_NOTHING_RETURN_SOURCE_AND_SETTINGS_LIST:
                        if (input != null) {
                            result.putExtra(ImgLyIntent.SOURCE_IMAGE_PATH, isFileSchema(input) ? input.getPath() : input.toString());
                            result.putExtra(ImgLyIntent.SOURCE_IMAGE_URI, input);
                        }

                        break;
                    default:
                        throw new RuntimeException("Unsupported save policy");
                }

                runOnUi(new ThreadUtils.MainThreadRunnable() {
                    @Override
                    public void run() {
                        getStateHandler().getStateModel(ProgressState.class).notifyExportFinish();
                        setResult(RESULT_OK, result);
                        finish();
                    }
                });
            }
        });
    }

    private boolean isFileSchema(Uri uri) {
        return "file".equals(uri.getScheme());
    }

    @NonNull
    protected LayerListSettings getLayerListSettings() {
        if (this.layerListSettings == null) {
            StateHandler stateHandler = getStateHandler();
            this.layerListSettings = stateHandler.getStateModel(LayerListSettings.class);
        }
        return this.layerListSettings;
    }

    private void deselectSticker() {
        getLayerListSettings().setSelected(null);
    }

    @MainThread
    @OnEvent(value = IMGLYEvents.EditorShowState_LAYER_TOUCH_END)
    void onLayerTouchEnd() {

        if (brushSettings.isInEditMode) {
            historyState.save(0, BrushSettings.class);
            brushPanel.onBrushEnd();
        }
        canvasTouched = false;
    }

    @OnEvent(value = ly.img.android.IMGLYEvents.EditorShowState_LAYER_DOUBLE_TAPPED, doInitCall = false)
    public void onStickerDoubleTapped(){
        if (layerListSettings != null && (layerListSettings.getSelected() instanceof TextLayerSettings)) {
            switchEditorMode(EDITOR_MODE_EDIT_TEXT);
        }
    }

    @MainThread
    @OnEvent(value = ly.img.android.IMGLYEvents.EditorShowState_LAYER_TOUCH_START)
    void onLayerTouchStart() {
        if (!canvasTouched) {
            canvasTouched = true;
        }
    }

    @Override
    public void onTimeOut(Enum identifier) {
        // Animation for UI elements
        if (editorMode == EDITOR_MODE_MENU && layerListSettings != null &&
                (layerListSettings.getSelected() instanceof TextLayerSettings ||
                        layerListSettings.getSelected() instanceof ImageStickerLayerSettings)) {
            if (canvasTouched) {
                if (animatorSet != null && animatorSet.isRunning()) {
                    animatorSet.cancel();
                }
                animatorSet = new AnimatorSet();
                animatorSet.playTogether(
                        ObjectAnimator.ofFloat(menuPanel, "alpha", 1f, 0f)
                );
                animatorSet.setDuration(ANIMATION_DURATION);
                animatorSet.start();
            } else {
                float alpha = menuPanel.getAlpha();

                if (animatorSet != null && animatorSet.isRunning()){
                    animatorSet.cancel();
                }
                animatorSet = new AnimatorSet();
                animatorSet.playTogether(
                        ObjectAnimator.ofFloat(menuPanel, "alpha", alpha, 1f)
                );
                animatorSet.setDuration(ANIMATION_DURATION / 3);
                animatorSet.start();
            }
        }
    }

    public void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        if (requestCode == SELECT_IMAGE) {
            if (resultCode == Activity.RESULT_OK) {
                if (data != null) {
                    try {
                        Bitmap bitmap =
                                MediaStore.Images.Media.getBitmap(getContentResolver(), data.getData());

                        uploadLogo(persistImage(bitmap, "tempory_logo"));
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
                }
            }
        }
    }

    private File persistImage(Bitmap bitmap, String name) {
        File filesDir = getApplicationContext().getFilesDir();
        File imageFile = new File(filesDir, name + ".png");

        OutputStream os;
        try {
            os = new FileOutputStream(imageFile);
            bitmap.compress(Bitmap.CompressFormat.PNG, 100, os);
            os.flush();
            os.close();
        } catch (Exception e) {
            Log.e(getClass().getSimpleName(), "Error writing bitmap", e);
        }
        return imageFile;
    }

    private ProgressDialog progressDialog;

    private void uploadLogo(File file) {
        progressDialog = new ProgressDialog(this);
        progressDialog.setMessage("Enviando logo...");
        progressDialog.setCancelable(false);
        progressDialog.show();

        RequestBody requestBody = RequestBody.create(MediaType.parse("*/*"), file);
        MultipartBody.Part fileToUpload = MultipartBody.Part.createFormData("logo", file.getName(),
                requestBody);

        ApiService getResponse = ApiClient.getClient().create(ApiService.class);
        Call<LogoToResponse> call = getResponse.addLogo("5dbbf86f318f9f0001399d4e", fileToUpload);
        call.enqueue(new Callback<LogoToResponse>() {
            @Override
            public void onResponse(Call<LogoToResponse> call, Response<LogoToResponse> response) {
                LogoToResponse logoToResponse = response.body();
                if (logoToResponse != null) {
                    Toast.makeText(getApplicationContext(),
                            "Enviado com sucesso!",Toast.LENGTH_SHORT).show();

                    List<CustomImageStickerAsset> stickerData = Utils.Companion.retrieveAssets(
                            logoToResponse.getLogos(), InstagramUIActivity.this);

                    // Update logo panel
                    logoPanel.setAdapter(stickerData);
                } else {
                    Log.v("Response", "Error to send");
                }
                progressDialog.dismiss();
            }

            @Override
            public void onFailure(Call<LogoToResponse> call, Throwable t) {
            }
        });
    }

}